import 'dart:convert';

import '../../didl.dart';
import '../../dlna_action_result.dart';
import '../../dlna_device.dart';
import '../position_info.dart';
import '../soap_action.dart';
import 'play.dart';
import 'stop.dart';

class SetUrl extends AbsDLNAAction<String> {
  DIDLObject didlObject;
  Function(PositionInfo positionInfo) progress;

  SetUrl(DLNADevice dlnaDevice, DIDLObject didlObject) : super(dlnaDevice) {
    this.didlObject = didlObject;
  }

  @override
  Future<DLNAActionResult<String>> execute() async {
    if (dlnaDevice.isXiaoMiDevice) {
      await Stop(dlnaDevice).execute();
    }
    var result = await start();
    if (result.success) {
      result.result = result.httpContent;
      await Play(dlnaDevice).execute();
      if (didlObject.refreshPosition) {
      }
    }
    return result;
  }

  @override
  String getControlURL() {
    return dlnaDevice.description.avTransportControlURL;
  }

  @override
  String getXmlData() {
    var time = DateTime.fromMillisecondsSinceEpoch(
        DateTime.now().millisecondsSinceEpoch);
    var title = HtmlEscape().convert(didlObject.title);
    var url = HtmlEscape().convert(didlObject.url);
    return """<?xml version='1.0' encoding='utf-8' standalone='yes' ?>
<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">
<s:Body>
<u:SetAVTransportURI xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">
<InstanceID>0</InstanceID>
<CurrentURI>${url}</CurrentURI>
<CurrentURIMetaData>
  <DIDL-Lite xmlns="urn:schemas-upnp-org:metadata-1-0/DIDL-Lite/" xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:upnp="urn:schemas-upnp-org:metadata-1-0/upnp/" xmlns:dlna="urn:schemas-dlna-org:metadata-1-0/">
	  <item id="id" parentID="0" restricted="0">
		  <dc:title>${title}</dc:title>
		  <upnp:artist>unknow</upnp:artist>
      <upnp:class>object.item.videoItem</upnp:class>
		  <dc:date>${time}</dc:date>
		  <res protocolInfo="${didlObject.protocol}">${url}</res>
	  </item>
  </DIDL-Lite>
</CurrentURIMetaData>
</u:SetAVTransportURI>
</s:Body>
</s:Envelope>""";
  }

  @override
  String getSoapAction() {
    return '\"urn:schemas-upnp-org:service:AVTransport:1#SetAVTransportURI\"';
  }
}
