import 'dart:convert';
import 'dart:io';

import 'package:xml2json/xml2json.dart';

import '../dlna_device.dart';

final DEVICE_TYPE = "deviceType";
final UDN = "UDN";
final FRIEND_NAME = "friendlyName";
final MANUFACTURER = "manufacturer";
final MANUFACTURER_URL = "manufacturerURL";
final MODEL_DESCRIPTION = "modelDescription";
final MODEL_NAME = "modelName";
final MODEL_URL = "modelURL";
final SERVICE = "service";
final SERVICE_TYPE = "serviceType";
final SERVICE_ID = "serviceId";
final CONTROL_URL = "controlURL";
final EVENT_SUB_URL = "eventSubURL";
final SCPDURL = "SCPDURL";

final X_RCONTROLLER_DEVICEINFO = "x_rcontroller_deviceinfo";
final X_RCONTROLLER_VERSION = "x_rcontroller_version";
final X_RCONTROLLER_SERVICE = "x_rcontroller_service";
final X_RCONTROLLER_SERVICETYPE = "x_rcontroller_servicetype";
final X_RCONTROLLER_ACTIONLIST_URL = "x_rcontroller_actionlist_url";

final AV_X_RCONTROLLER_DEVICEINFO = "av:x_rcontroller_deviceinfo";
final AV_X_RCONTROLLER_VERSION = "av:x_rcontroller_version";
final AV_X_RCONTROLLER_SERVICE = "av:x_rcontroller_service";
final AV_X_RCONTROLLER_SERVICETYPE = "av:x_rcontroller_servicetype";
final AV_X_RCONTROLLER_ACTIONLIST_URL = "av:x_rcontroller_actionlist_url";

final AV_TRANSPORT = "urn:schemas-upnp-org:service:AVTransport:1";
final RENDERING_CONTROL = "urn:schemas-upnp-org:service:RenderingControl:1";
final CONNECTION_MANAGER = "urn:schemas-upnp-org:service:ConnectionManager:1";

class DescriptionParser {
  var httpClient = HttpClient();

  Future<DLNADescription> getDescription(DLNADevice dlnaDevice) async {
    String url = dlnaDevice.location;
    var request = await httpClient.getUrl(Uri.parse(url));
    var response = await request.close();
    var responseBody = await response.transform(utf8.decoder).join();
    final myTransformer = Xml2Json();
    myTransformer.parse(responseBody);
    String json = myTransformer.toParker();
    var source = jsonDecode(json);

    DLNADescription description = DLNADescription();
    var root = source['root'];
    var device = root['device'];

    description.deviceType = device[DEVICE_TYPE];
    description.friendlyName = device[FRIEND_NAME];
    description.udn = device[UDN];
    description.manufacturer = device[MANUFACTURER];
    description.manufacturerURL = device[MANUFACTURER_URL];
    description.modelDescription = device[MODEL_DESCRIPTION];
    description.modelName = device[MODEL_NAME];
    description.modelURL = device[MODEL_URL];

    var serviceList = device['serviceList'];
    if (serviceList != null) {
      var service = serviceList[SERVICE];
      if (service is List) {
        var dlnaServices = List<DLNAService>();
        for (var item in service) {
          var dlnaService = DLNAService();
          dlnaService.type = item[SERVICE_TYPE];
          dlnaService.serviceId = item[SERVICE_ID];
          dlnaService.SCPDUrl = item[SCPDURL];
          dlnaService.controlUrl = item[CONTROL_URL];
          dlnaService.eventSubUrl = item[EVENT_SUB_URL];
          dlnaServices.add(dlnaService);
          if (AV_TRANSPORT == dlnaService.type) {
            description.avTransportControlURL = dlnaService.controlUrl;
          } else if (RENDERING_CONTROL == dlnaService.type) {
            description.renderingControlControlURL = dlnaService.controlUrl;
          } else if (CONNECTION_MANAGER == dlnaService.type) {
            description.connectionManagerControlURL = dlnaService.controlUrl;
          }
        }
        description.dlnaServices = dlnaServices;
      }
    }
    return description;
  }

  void stop() {
    httpClient.close();
  }
}
